; ********************************************
; DRIVERS FOR:
;
; ACT II (Advanced Color Technology)
; ********************************************
;
IF prn_mdl eq MDL_CHROMAJET
	;	ACT II ChromaJet driver
	;
	;	Documentation supplied by:
	;		Donald W. Campbell
	;		1913 S.	5th Ave.
	;		Marshalltown, IA 50158
	;		515/752-4038

	prn_byte	= byte_horiz
	PRN_GR		= GR_FIX + GR_RUN
	prn_chrset	= chrset_ascii7	; Standard PC-8	-> ASCII

	GR_RUN_MAX	EQU 256		; maximum run length
	GR_RUN_OFFSET	EQU 0		;  count=1 produces 1 byte, count=0 is 256.

	;	Create GMODES table using GFXMAP macro.

	;	 prn			(1/10")
	;      # mode  rep rot pattern	x"   y"
	GFXMAP 0,  0,	2,  0,	   200,	80, 105	; 320x200
	GFXMAP 1,  0,	2,  0,	    10,	80, 105	; 640x200
	GFXMAP 2,  0,	1,  1,	     6,	80, 105	; 640x350
	GFXMAP 3,  0,	1,  0,	     8,	80, 105	; 640x480
	GFXMAP 4,  0,	1,  1,	   200,	80, 105	; 720x348
	GFXMAP X,  0,	1,  0,	     8,	80, 105
	; Alternates available
	GFXMAP 10, 0,	1,  1,	     6,	80, 105	; 320x200
	GFXMAP 11, 0,	1,  1,	     3,	80, 105	; 640x200
	GFXMAP 12, 0,	1,  0,	    11,	80, 105	; 640x350
	GFXMAP 13, 0,	1,  1,	     8,	80, 105	; 640x480
	GFXMAP 14, 0,	1,  1,	   200,	80, 105	; 720x348
	GFXMAP 1X, 0,	1,  1,	     8,	80, 105
	gfxmap_end label word

	;	Graphics Mode definitions.

	GSET_PER_MODE	EQU	TRUE	; Setup	strings	for each graphic mode
	GMODES	DW	GSET
		PRNMODE	0, 80, 80
	MAXMODE	EQU	0

	; The color lookup table is sent before all graphics modes.
	GSET_ALL	DW	GSET_ALL_END-GSET_ALL_START
	GSET_ALL_START LABEL BYTE
		DB	01h	; Define table lookup
		db	64,0	; Lookup size is 64
		; Define EGAD's 64 colors.
		;
		;
		;
		;
		;
	GSET_ALL_END LABEL BYTE

	; The different graphics setups set the aspect ratio (gfx resolution)
	GSETUP	DW	GSET320X200, GSET640X200, GSET640X350, GSETSQUARE
	; Each consists of:
	;	3	Enter graphics mode
	;	2	Set image width and max length
	;	{width}  (width and height are LSB first)
	;	{height} NOTE: We send ALPHA code to 'bail out' before this!
	;	0eh	ZOOMX
	;	{xzoom}  number of printer pixels used per data point (1..8)
	;	0Fh	ZOOMY
	;	{yzoom}  as ZOOMX but in Y direction
	;	18h	ASPECT: Actually sets the horizontal resolution
	;	{aspect} where 2=100dpi, 3=85.5dpi, 4=75dpi, 5=66.77dpi, 6=60dpi
	EGADSTR GSET320X200,<3,2,40h,01h,00h,03h,0eh,02h,0fh,03h,18h,03h>
	EGADSTR GSET640X200,<3,2,80h,02h,00h,03h,0eh,02h,0fh,03h,18h,03h>
	EGADSTR GSET640X350,<3,2,80h,02h,00h,03h,0eh,02h,0fh,02h,18h,


	EGADSTR	PMODE_PRT0,<>			; Already In Color Scan	Mode
	PMODE_LEN0	DW	80		; 640 dots/line	= 80 bytes

	EGADSTR	GR_OFF,<>	; n/a -	can change colors in gfx mode.

	; Enter	graphics mode. send picture size.
	GSETUP	DW	 GSETUP_END - GSETUP_START
	GSETUP_START LABEL BYTE
		DB     3	    ; Start graphics mode
		DB     2	    ; Picture size command
		DB     080h,002h    ; 640 dots/line
		DB     0ffh,
	GSETUP_END LABEL BYTE
	EGADSTR	GSETUP,<003h,002h,>

	EGADSTR	MLCRLF,<>			; no CR	codes needed

	EGADSTR	NSPACE,<>

	; Position @ dot 0 (DOES NOT WORK EVEN THO' MANUAL SAYS	IT WILL!!!)
	EGADSTR	CR,<13>

	; CR alone does	CR & LF	functions
	EGADSTR	CRLF,<13,10>

	NBUFFERS EQU 1

	;	Text mode, Attribute definitions.
	;	Treat colors like text attributes.

	TEXT_ATTRS LABEL WORD
		DW	T_BLACK		; 0
		DW	T_YELLOW	; 1
		DW	T_MAGENTA	; 2
		DW	T_RED		; 3
		DW	T_CYAN		; 4
		DW	T_GREEN		; 5
		DW	T_BLUE		; 6
		DW	T_WHITE		; 7

	N_TEXT_ATTRS EQU (($-TEXT_ATTRS)/2)-1 ; should be 7

	EGADSTR T_WHITE,   <17,7,16,0>	; White characters on black bkg.
	EGADSTR T_BLACK,   <17,0,16,7>	; Black characters on white
	EGADSTR T_MAGENTA, <17,0,16,2>
	EGADSTR T_RED,     <17,0,16,3>
	EGADSTR T_YELLOW,  <17,0,16,1>
	EGADSTR T_GREEN,   <17,0,16,5>
	EGADSTR T_CYAN,    <17,0,16,4>
	EGADSTR T_BLUE,    <17,0,16,6>

	; XMIT_NOCHG is not defined, as we must send either one or the
	; other of these codes to begin transmission of graphics data.
	EGADSTR	XMIT_NORM,<007h>	; Normal transmission mode
	EGADSTR	XMIT_RUNL,<01Dh>	; Run-length mode

	PATTERN	200, 1, 1
		db	11111111b	; Just 1 pattern
		db	0

	dither_levels	equ  255	; 256 won't fit into byte!
	dither_width	equ    1
	ndithers	equ    1

	; Page print width = 8". Maximum dot density=100 dpi. 1 pixel/byte.
	; Maximum buffer size: 800, plus 2 bytes of length.
	BUFLEN	EQU	802

	DITHER_TABLE	LABEL WORD
	COLOR_XLAT	LABEL BYTE
		; PostScript colors are: ccmmyykk
		; "Abstract" colors    : --yymmcc
		; 0-63 analogues:
		db	000h,040h,080h,0C0h,010h,050h,090h,0D0h
		db	020h,060h,0A0h,0E0h,030h,070h,0B0h,0F0h
		db	004h,044h,084h,0C4h,014h,001h,094h,0D4h
		db	024h,064h,0A4h,0E4h,034h,074h,0B4h,0F4h
		db	008h,048h,088h,0C8h,018h,058h,098h,0D8h
		db	028h,068h,002h,0E8h,038h,078h,0B8h,0F8h
		db	00Ch,04Ch,08Ch,0CCh,01Ch,05Ch,09Ch,0DCh
		db	02Ch,06Ch,0ACh,0ECh,03Ch,07Ch,0BCh,003h
ENDIF
