; ********************************************
; DRIVERS FOR:
;
; HP PCL Printers
;	HP QuietJet
; PCL-3:		{Roman-8 character set only}
;	HP LaserJet
;	HP LaserJet +
; PCL-4:		{PC-8 character set available}
;	HP LaserJet II
; PCL-5:		{Data compression used}
;			{NOTE: Will work with many printers which are actually
;			 only PCL-4 since we use only relatively simple features.}
;	HP LaserJet III
;	HP PaintJet
;	HP DeskJet 500, 500C, 550C, 520, 560
; ********************************************
; 

	; All supported HP printers have at least PCL 3
	PCL_LEV	= 3
	PRN_CHRSET = CHRSET_ROMAN8	; HP default set

	IF (PRN_MDL GE MDL_HPPCL4)
		PCL_LEV = 4
		PRN_CHRSET = CHRSET_PC8	; IBM PC set available
	ENDIF
	IF (PRN_MDL GE MDL_HPPCL5)
		PCL_LEV = 5
	ENDIF

	PRN_BYTE	= BYTE_HORIZ
 IF PCL_LEV GE 5
	; Enable run-length encoding for HP Series III, PaintJet, etc.
	PRN_GR		EQU	GR_DEC + GR_RUN
	GR_DEC_DIGITS	EQU	3
	PRINT_ALL	EQU	TRUE	; Can print all characters using quote

	GR_RUN_MAX	EQU 256		; maximum run length
	GR_RUN_OFFSET	EQU -1		;  count=0 produces 1 byte
 ELSE
	PRN_GR		EQU	GR_DEC
	GR_DEC_DIGITS	EQU	3
 ENDIF
	HORIZ_BYTES	EQU	TRUE
	GSET_PER_MODE	EQU	TRUE	; Setup strings for each graphic mode

	;
	;	GMAP: Table of screen resolutions to printer resolutions.
	;	(** See comment below about REPEAT field on HP printer **)
	;
	;	X & Y are limits of full page enlargement.
	;
	;	Printer Horizontal: Resolution (dpi) * Screen Pixels/Byte
	;		[* 8 inches / 8 bit/bytes -- these cancel]
	;	Printer Vertical: Resolution (dpi) * 10 Inches /
	;		Repeat Factor

	;	Graphics Mode definitions.
	;

	PMODE_PRT0	LABEL WORD
	PMODE_PRT1	LABEL WORD
	PMODE_PRT2	LABEL WORD
			DW 7
			DB	27,'*b'
	PMODE_LEN0	LABEL	BYTE
	PMODE_LEN1	LABEL	BYTE
	PMODE_LEN2	LABEL	BYTE
			DB	gr_dec_digits dup (0)	; Count goes here
  IF (PRN_TYP AND TYP_COLOR)
			DB	'V'	; Send one scan line of color
  ELSE
			DB	'W'	; Send one scan line (B&W)
  ENDIF

 IF (PRN_MDL EQ MDL_HPQJ) ; QuietJet
	GMODES	LABEL	WORD
		PRNMODE	0,  96,  96
		PRNMODE	1, 192, 192
		PRNMODE	2, 192, 192
	MAXMODE	EQU	2

	; Page print width is eight inches. Maximum dot density is
	; RESX1. 8 Bits/Byte. Maximum buffer size RESX2, plus 2 bytes length.
	BUFLEN	EQU	RESX2 + 2

	; QuietJet resolutions:
	GSETUP	DW	GSET768,GSET1536,GSET1536
	; Setup: 96 dpi (768 dots across)
	EGADSTR GSET768,<27,'*t96R',27,'*rA'>
	; Setup: 192 dpi (1536 dots across)
	EGADSTR GSET1536,<27,'*t192R',27,'*rA'>	; Set resolution & begin gfx
 ELSE
  IF (PRN_MDL EQ MDL_HPPJ) ; PaintJet
	GMODES	LABEL	WORD
		PRNMODE	0,  90,  90
		PRNMODE	1, 180, 180
		PRNMODE	2, 180, 180
	MAXMODE	EQU	2
	BUFLEN	EQU	RESX2 + 2
	NBUFFERS EQU	3

	GSETUP	DW	GSET720, GSET1440, GSET1440
	EGADSTR GSET720,<27,'*t90R'>	; Set 90 dpi
	EGADSTR GSET1440,<27,'*t180R'>	; Set 180 dpi

	GSET_ALL	DW	GSET_ALL_END-GSET_ALL_START
	GSET_ALL_START LABEL BYTE
		DB	27,'*r1440s'	; Set 8" width
		DB	27,'*r3U'	; Sending 3 planes

		DB	27,'*v90A'	; Set palette #0 to white (90,88,85)
		DB	27,'*v88B'
		DB	27,'*v85C'
		DB	27,'*v0I'

		DB	27,'*v2A'	; Set palette #1 to cyan (2,22,64)
		DB	27,'*v22B'
		DB	27,'*v64C'
		DB	27,'*v1I'

		DB	27,'*v53A'	; Set palette #2 to magenta (53,5,25)
		DB	27,'*v5B'
		DB	27,'*v25C'
		DB	27,'*v2I'

		DB	27,'*v4A'	; Set palette #3 to blue (4,4,29)
		DB	27,'*v4B'
		DB	27,'*v29C'
		DB	27,'*v3I'

		DB	27,'*v89A'	; Set palette #4 to yellow (89,83,13)
		DB	27,'*v83B'
		DB	27,'*v13C'
		DB	27,'*v4I'

		DB	27,'*v3A'	; Set palette #5 to green (3,26,22)
		DB	27,'*v26B'
		DB	27,'*v22C'
		DB	27,'*v5I'

		DB	27,'*v53A'	; Set palette #6 to red (53,8,14)
		DB	27,'*v8B'
		DB	27,'*v14C'
		DB	27,'*v6I'

		DB	27,'*v4A'	; Set palette #7 to black (4,4,6)
		DB	27,'*v4B'
		DB	27,'*v6C'
		DB	27,'*v7I'

		DB	27,'*r0A'	; Start raster graphics at left margin
		DB	27,'*b0M'	; Non-compressed data
	GSET_ALL_END LABEL BYTE
  ELSE
	; LaserJet, DeskJet, DeskJet 500 & 500C resolutions:
	GMODES	LABEL	WORD
		PRNMODE 0, 100, 100
		PRNMODE	1, 150, 150
		PRNMODE	2, 300, 300
	MAXMODE	EQU	2
	BUFLEN	EQU	RESX2 + 2

	GSETUP	DW	GSET800,GSET1200,GSET2400

	EGADSTR GSET800,<27,'&l0O',27,'*t100R'>  ; 100 dpi
	EGADSTR GSET1200,<27,'&l0O',27,'*t150R'> ; 150 dpi
	EGADSTR GSET2400,<27,'&l0O',27,'*t300R'> ; 300 dpi
	; Esc-&l0O sets Portrait mode

	GSET_ALL	DW	GSET_ALL_END-GSET_ALL_START
	GSET_ALL_START LABEL BYTE
		DB	27,'&l0L'	; Perf skip off
   IF (PRN_MDL EQ MDL_HPDJ500) OR (PRN_MDL EQ MDL_HPDJ550)
		; DeskJet and DeskJet 500C specific code:
		DB	27,'*p4N'	; Smart bi-directional print
		DB	27,'*o1D'	; No "depletion"
    IF (PRN_MDL EQ MDL_HPDJ550)
		DB	27,'*o0M'	; Normal Print Quality, as per
					; HP Peripherals News May '94
    ENDIF
    IF (PRN_TYP AND TYP_COLOR)		; DeskJet 500C:
      IF (PRN_MDL EQ MDL_HPDJ550)
		DB	27,'*b1B'	; Enable gray balancing, as per
					; HP Peripherals News May '94
	NBUFFERS	EQU 4		; Will do 4-color CMYK, color + black.
		DB	27,'*o1Q'	; 50% shingling
		DB	27,'*r0Q'	;   Keypad quality setting
		DB	27,'*r-4U'	;   4-plane CMYK
      ELSE
	NBUFFERS	EQU 3		; No black buffer!
		DB	27,'*o0Q'	; No "shingling"
		DB	27,'*r0Q'	;   Keypad quality setting
		; DB	27,'*r1Q'	;   Draft quality
		DB	27,'*r-3U'	;   3-plane CMY
      ENDIF
    ELSE				; DeskJet 500:
		DB	27,'*o0Q'	; No "shingling"
		DB	27,'*r0Q'	;   Keypad setting: draft/high quality
		DB	27,'*r0U'	;   1 plane palette
    ENDIF
   ENDIF ; if DeskJet 5xx
		DB	27,'*r0A'	; Begin raster graphics @ left margin
	GSET_ALL_END LABEL BYTE
  ENDIF ; if paintjet
 ENDIF ; if quietjet

	;        prn                    (1/10")
	;      # mode  rep rot pattern  x"   y"
	; Normal (Best Ratio)
	GFXMAP 0,  0,   2,  0,     5, 80, 103	; 320x200 5.1x4.0" P
	GFXMAP 1,  1,   3,  0,     7, 80, 103	; 640x200 5.3x6.4" P
	GFXMAP 2,  1,   2,  1,     3, 80, 103	; 640x350 8.5x6.2" L
	GFXMAP 3,  1,   2,  1,     4, 80, 103	; 640x480 8.5x6.4" L
	GFXMAP 4,  1,   2,  0,     6, 80, 103	; 720x348 6.4x4.6" P
   IF RESX2 GE 200
	GFXMAP X,  2,   2,  0,     4, 80, 103	; Reduce (300 dpi)
   ELSE
	GFXMAP X,  2,   1,  0,	   8, 80, 103	; Reduce (192 or 180 dpi)
   ENDIF
	; Alternates
	GFXMAP 10, 1,   3,  1,	   2, 80, 103	; 320x200 5.3x6.4" L
	GFXMAP 11, 1,   1,  1,	   3, 80, 103	; 640x200 3.6x4.3" L
	GFXMAP 12, 1,   2,  0,	   6, 80, 103	; 640x350 6.8x4.7" P
	GFXMAP 13, 0,   1,  0,	   8, 80, 103	; 640x480 6.4x4.8" P
	GFXMAP 14, 1,   1,  1,	   5, 80, 103	; 720x348 4.8x3.7" L
   IF RESX2 GE 200
	GFXMAP 1X, 2,   2,  1,	   4, 80, 103	; Reduce (300 dpi)
   ELSE
	GFXMAP 1X, 2,   1,  1,	   8, 80, 103	; Reduce (192 or 180 dpi)
   ENDIF
	GFXMAP_END LABEL BYTE

 ; End raster graphics transfer:
 IF (PRN_TYP AND TYP_COLOR) NE 0
	EGADSTR	NSPACE,<27,'*rB',27,'&l1l3E',27,'*r0U'>
 ELSE
	EGADSTR	NSPACE,<27,'*rB',27,'&l1l3E'>
 ENDIF
		; *rB  end graphics
		; &l1l Perf skip on
		;   3E Set top margin to 3
		; *r0U Monochrome graphics
	EGADSTR GR_OFF,<>		; No end-graphics code
	EGADSTR	CR,<13>			; CR only
	EGADSTR CRLF,<13,10>		; CR/LF pair
	EGADSTR PRINTER_RESET,<27,'E',27,'&l2E'>
		; Reset-printer code
		; (Reset sent before beginning job if desired.)
		; &l2E  -- sets top margin to 2 lines down
		;          (cursor will be 75% of 8/48" down from this.)

 IF PCL_LEV GE 4
	EGADSTR QUOTE_CHAR,<27,'&p1X'>
 ENDIF

 IF (PRN_TYP AND TYP_COLOR) NE 0
	EGADSTR MLCRLF,<27,'*b0W'>	; Extra (ignored) plane forces printout
					; and paper advance.

	GR_YELLOW LABEL WORD
	GR_MAGENTA LABEL WORD
	GR_CYAN LABEL WORD
  IF (NBUFFERS EQ 4)
	GR_BLACK LABEL WORD		; Only define if separate layer (DJ 550)
  ENDIF
		DW	0		; No color-select codes in graphics mode

 ELSE
	EGADSTR MLCRLF,<> 		; Nothing sent between gfx rows
 ENDIF

	; All PCL printers now support text attributes.
	; Color text support disabled.
	TEXT_ATTRS LABEL WORD
		DW	text_nor	; 0
		DW	text_bold	; 1
		DW	text_undr	; 2
		DW	text_ital	; 3

	N_TEXT_ATTRS EQU (($-TEXT_ATTRS)/2)-1 ; normally 3

	EGADSTR TEXT_NOR, <27,'(s0p10h12v0s0b3t2Q',27,'&d@'>
	EGADSTR TEXT_BOLD,<27,'(s3B'>
	EGADSTR TEXT_UNDR,<27,'&d0D'>
	EGADSTR TEXT_ITAL <27,'(s1S'>

	IF (PRN_GR AND GR_RUN)
	  EGADSTR XMIT_NORM,<27,'*b0M'>	; Normal transmission mode
	  EGADSTR XMIT_RUNL,<27,'*b1M'>	; Run-length mode
	  EGADSTR XMIT_NOCHG,<>		; No change in xmit mode
	ENDIF

	IF (PCL_LEV EQ 3)
	  EGADSTR EXT_CHARS,<27,'(8U'>	; Roman-8 character set
	ELSE
	  EGADSTR EXT_CHARS,<27,'(10U'>	; PC-8 character set
	ENDIF

	;	HP chose to have their bytes run HORIZONTALLY instead
	;	of VERTICALLY. Raster graphic commands produce one
	;	horizontal line of dots.  The MSB is on the left of
	;	each byte.  (The REPEAT field allows us to dither
	;	into multiple print rows.)

	PATTERN 2, 2, 8		; 2 Screen Lines per print column
	PATTERN_200 EQU PATTERN_2
		db	11110000b
		db	00001111b
	PATTERN	3, 3, 8		; 3 Screen Lines per print column
		db	11000000b
		db	00111000b
		db	00000111b
	PATTERN 4, 4, 8		; 4 Screen Lines per print column
		db	11000000b
		db	00110000b
		db	00001100b
		db	00000011b
	PATTERN	5, 5, 8		; 5 Screen Lines per print column
		db	10000000b
		db	01100000b
		db	00011000b
		db	00000100b
		db	00000011b
	PATTERN	6, 6, 8		; 6 Screen Lines per print column
		db	10000000b
		db	01100000b
		db	00010000b
		db	00001000b
		db	00000110b
		db	00000001b
	PATTERN	7, 7, 8		; 7 Screen Lines per print column
		db	10000000b
		db	01000000b
		db	00100000b
		db	00010000b
		db	00001000b
		db	00000110b
		db	00000001b
	PATTERN	8, 8, 8		; 8 Screen Lines per print column
		db	10000000b
		db	01000000b
		db	00100000b
		db	00010000b
		db	00001000b
		db	00000100b
		db	00000010b
		db	00000001b
		db	0		; end of patterns

  IF (PRN_TYP AND TYP_COLOR) NE 0
	; 4 Dither levels, but 2 dither tables (entries taken from
	; the B&W printer's dither tables.)
	DITHER_LEVELS	EQU 4			; # of dither patterns
	DITHER_WIDTH	EQU 1			; 1 byte per level
	NDITHERS	EQU 4			; 4 dither tables

	DITHER_TABLE LABEL WORD
		DB	000H,022H,055H,0FFH	; 1st line
		DB	000H,011H,0AAH,0FFH	; 2nd line
		DB	000H,044H,055H,0FFH	; 3rd line
		DB	000H,088H,0AAH,0FFH	; 4th line
  ELSE
	; 16 dither patterns: 8 shades & 8 patterns
	DITHER_LEVELS	EQU 16
	DITHER_WIDTH	EQU 1
	NDITHERS	EQU 4			; 4 dither tables

	DITHER_TABLE	LABEL WORD
		; 1st line byte patterns
		DB	000H,020H,022H,044H,0CCH,055H,0DDH,0FFH
		DB	0FFH,0AAH,080H,0CCH,0CCH,033H,0C3H,03CH
		; 2nd line
		DB	000H,002H,088H,0AAH,033H,0AAH,077H,0FFH
		DB	000H,0AAH,080H,0CCH,099H,099H,00FH,0C3H
		; 3rd line
		DB	000H,008H,022H,011H,0CCH,055H,0DDH,0FFH
		DB	000H,0AAH,080H,0CCH,033H,0CCH,03CH,0C3H
		; 4th line
		DB	000H,080H,088H,0AAH,033H,0AAH,077H,0FFH
		DB	0FFH,0AAH,080H,0CCH,066H,066H,0F0H,03CH
  ENDIF
  IFDEF NBUFFERS
    IF (NBUFFERS EQ 4)
	COLOR_XLAT	LABEL BYTE
		; 4-color CMYK levels  : yymmcckk
		; "Abstract" colors    : --yymmcc
		db 000h,004h,008h,00Ch,010h,014h,018h,01Ch
		db 020h,024h,028h,02Ch,030h,034h,038h,03Ch
		db 040h,044h,048h,04Ch,050h,001h,058h,009h
		db 060h,064h,068h,06Ch,070h,021h,078h,029h
		db 080h,084h,088h,08Ch,090h,094h,098h,09Ch
		db 0A0h,0A4h,002h,006h,0B0h,0B4h,012h,016h
		db 0C0h,0C4h,0C8h,0CCh,0D0h,081h,0D8h,089h
		db 0E0h,0E4h,042h,046h,0F0h,0A1h,052h,003h
    ENDIF
  ENDIF

	; End of Hewlett-Packard printer support
