;	PAGE	82,132
	PAGE	84,160
	TITLE	EGAD Screen Print v2.50.3 (c) 1994 LS Software
	SUBTTL	EGAD.ASM -- Main Program
;
;	COPYRIGHT (C) 1988-1993, 1994 LS SOFTWARE
;
;	EGAD is a screen dump utility. It intercepts INT 5 from
;	the BIOS, and dumps the contents of the screen onto a printer.
;
; This program calls a subroutine which shows a box around the sereen.
; The operator may press return to print the entire screen, or use the
; arrow keys to select the area of the screen to print.  If the print
; screen key was pressed in error, the escape key can be used to exit
; without printing anything.   Where it is desired to print a small
; graphics area, pressing the 2, 3, or 4 key will expand the print out
; by a ratio of 2, 3, or 4.
;
; When this program is executed by the DOS system, it displays a message
; showing its identity and verify that the intended operator action
; had the desired effect.
;
; Next it checks to see that it was not previously installed.
; If it was, a message is displayed and the program exits without
; doing anything.
;
; If this is the first execution of the program, The old interrupt 5
; vector is saved and a new vector for interrupt 5 that points to the
; resident portion of this program is put in place.
;
; The space needed for the resident portion of the program is computed
; and the program exits to DOS with a request to stay resident.
;
; At any time interrupt 5 is activated, the resident portion of this
; program is executed.
;
; As this program has no way of knowing how much stack space is available
; at the time of activation, this program uses a local stack that is known
; to be large enough for all its operations. The user's stack segment and
; stack pointer are saved.
;
; When action is complete, the user stack is restored and
; an interrupt return is executed.
;
	PUBLIC	INT_5,OLD_INT_5,EXEHDR,CMD_POINTER,GET_NEXT
	PUBLIC	ERRCOD, START
	PUBLIC	INT_SS,INT_SP,INT_STACK
	EXTRN	DO_DUMP:PROC
	EXTRN	DEFAULT_PRN:WORD
	EXTRN	FORCE_MODE:BYTE

	INCLUDE GLOBAL.INC		; Define segments, &c.
	INCLUDE MANUFACT.INC

IFDEF OEM
; *** Uncomment one set of these:

;COMPANY_NAME	EQU <'K.E. Ayers & Associates'>
;CRC_VALUE	EQU 01250H

;COMPANY_NAME	EQU <'Antenna Design Associates'>
;CRC_VALUE	EQU 01b42H

;COMPANY_NAME	EQU <'INEGI'>
;CRC_VALUE	EQU 0fcb3H

;COMPANY_NAME	EQU <'K.E.Ayers & Associates'>
;CRC_VALUE	EQU 00000H

;COMPANY_NAME	EQU <'Frank Menkin'>
;CRC_VALUE	EQU 00000H

;COMPANY_NAME	EQU <'The Rocky Mountain Demo Company'>
;CRC_VALUE	EQU 02729H

;COMPANY_NAME	EQU <'System Dynamics'>
;CRC_VALUE	EQU 09b22H

;COMPANY_NAME	EQU <'T.B.S.P., Inc.'>
;CRC_VALUE	EQU 0285cH

;COMPANY_NAME	EQU <'TexaSoft'>
;CRC_VALUE	EQU 0936bH

;COMPANY_NAME	EQU <'Western Dairy Research, Inc.'>
;CRC_VALUE	EQU 0d5e3H

COMPANY_NAME	EQU <'Ingenieria Computarizada Integral'>
CRC_VALUE	EQU 0C2A2H

ELSE
; Companies licensed to distribute the TSR version:
;COMPANY_NAME	EQU <'CSC Insurance Marketing Systems'>
;CRC_VALUE	EQU 04FEAH

; Retail EGAD Screen Print
CRC_VALUE EQU 06701h
ENDIF



_DATA		SEGMENT WORD PUBLIC 'DATA'
	EXTRN	TSR:BYTE
	EXTRN	HDR_NVARS:WORD
_DATA		ENDS

; Token parsing definitions

TK_END_DATA = 0
TK_SWITCH   = 1
TK_NUMBER   = 2
TK_ALPHA    = 3
TK_MISC     = 4

; EGAD Operation Codes

OP_NONE  = 0
OP_TSR   = 1
OP_PRINT = 2
OP_REMOVE= 3

; DOS calls
DOS_SET_VEC  = 25H
DOS_GET_VEC  = 35H
DOS_FREE_MEM = 49H
DOS_EXIT     = 4CH

	INCLUDE VERSION.INC

	SUBTTL	Main Program (Data areas: ID, Palettes, SETEGAD parameters, TSR Data)
	PAGE +

	ASSUME	CS:_TEXT, DS:_DATA, SS:STACK, ES:_DATA

	EXTRN	KBD_SETUP:PROC, KBD_RESET:PROC
;
;	The _DATA segment declarations should be in EGADDRV.ASM,
;	as it depends on configuration information.  The symbols
;	should be PUBLIC there, and EXTRN here.  WL 10/02/90
;
_DATA	SEGMENT

	EXTRN	FLAG:BYTE, FLAG1:WORD, SIGNATURE:WORD
	EXTRN	PALETTE_2:WORD,PALETTE_4:WORD,PALETTE_16:WORD,PALETTE_TEXT:WORD
	EXTRN	INST_MCB:WORD

ERRCOD	DW	0
INT_SS	DW	0	; Saved	user stack registers
INT_SP	DW	0
_DATA	ENDS

;
;	-------------
;	Stack Segment
;	-------------
;
STACK	SEGMENT
	org	0
	DB	256 DUP	(?)	; changed, WL 12/07/92
INT_STACK LABEL BYTE		; Start	of internal stack
STACK	ENDS

	SUBTTL	Main Program (Interrupt 5 Handler)
	PAGE +
_TEXT SEGMENT

IFNDEF OEM
;
;	Interrupt 5 Handler.  Gains control when Shift-PrtSc is pressed.
;
INT_SIGNATURE EQU 424BH

INT_5:	jmp	short int_vec
OLD_INT_5 DD	0		; So it is possible to remove EGAD from memory
INT_ID:	DW	INT_SIGNATURE	; Signature for Interrupt Chain
	DB	0
	JMP short NO_RESET
	DB	7 DUP (0)
INT_VEC:
	PUSH	AX
	PUSH	DS
	PUSHF
	MOV	AX,_bios
	MOV	DS,AX
	ASSUME	DS:_bios
	MOV	AL,IS_BUSY
	CMP	BUSY_FLAG,AL	; Exit if currently printing
	JE	ABORT
	MOV	BUSY_FLAG,AL	; Show busy

	MOV	AX,_DATA
	MOV	DS,AX
	ASSUME	DS:_DATA

	MOV	INT_SP,SP	; Save user stack
	MOV	INT_SS,SS

IF 1
	MOV	AX,STACK
	MOV	SS,AX		; Set local stack
	MOV	SP,OFFSET STACK:INT_STACK
ELSE
;	wl 12/07/92 -- the Stack overwrites the command tail.
;	We do this by making a 128-byte stack segment, but require
;	SS to be based 80h bytes below its beginning, so it is
;	based at the command tail.

	mov	ax,STACK	; PSP's segment
	sub	ax,8h		; End of command tail
	mov	ss,ax
	mov	sp,100h		; Command tail + 128 byte stack segment
ENDIF

	STI			; Now allow interrupts to come in

	PUSH	ES
	PUSH	BX

	 CALL	FAR PTR RUN_DUMP	; FAR PTR because we need segment
					; registers set up differently.
					; Here, DS=CS.  But we need CS=_TEXT
					; in DO_DUMP.
	 mov	bx,ERRCOD		; Resultant error code
	 or	bx,bx
	 mov	bl,not_busy		; If no error, show 'not busy'
	 jz	PRTSC_OK
	 mov	bl,err_busy		; If error, show that too.
PRTSC_OK:
	 push	ds
	  mov	ax,_bios
	  mov	ds,ax
	ASSUME	DS:_bios
	  mov	byte ptr busy_flag,bl	; Show print complete (maybe error)
	 pop	ds
	ASSUME	DS:_DATA

	POP	BX
	POP	ES
	CLI
	MOV	SP,INT_SP
	MOV	SS,INT_SS	; Restore user stack
ABORT:	POPF
	POP	DS
	POP	AX
	IRET
NO_RESET: RET
ELSE
	; OEM
INT_5:
OLD_INT_5:
	DB	0
ENDIF
	SUBTTL	Main Program (RUN_DUMP)
	PAGE	+
;
;	Call to DO_DUMP, which draws crop box and/or calls Printer Driver.
;
;	Preserves: ALL registers
;
RUN_DUMP PROC FAR
	PUSH	DS
	PUSH	ES
	PUSH	BX
	PUSH	CX
	PUSH	DX
	PUSH	SI
	PUSH	DI
	MOV	AX,_DATA
	MOV	DS,AX

	CALL	NEAR PTR KBD_SETUP	; Set up keyboard
	CALL	NEAR PTR DO_DUMP	; Bring up crop box & print
	CALL	NEAR PTR KBD_RESET	; Reset keyboard

	POP	DI
	POP	SI
	POP	DX
	POP	CX
	POP	BX
	POP	ES
	POP	DS
	RET
RUN_DUMP ENDP
_TEXT ENDS
	SUBTTL	Main Program (Initialization Code)
	PAGE	+
;
;	Code beyond this point is in memory only during invocation.
;	It is purged when EGAD is TSR.
;
;	------------
;	Startup Code
;	------------
;
START_TEXT SEGMENT
	ASSUME	CS:START_TEXT, DS:DGROUP, SS:STACK, ES:DGROUP
START PROC NEAR
;	INT	3		; !!!
	MOV	AX,DS		; The system sets DS to the PSP.
	MOV	ES,AX		; Set ES to PSP

	MOV	CX,DGROUP
	MOV	DS,CX		; Establish correct DS

	CMP	SIGNATURE,CFG_LEN ; Ensure parameter block not corrupted
	JE	start_ok
start_corrupt:
	DISPLY	MSG_INT_ERR
	JMP	EXIT_START

start_ok:
	MOV	EXEHDR,AX	; Save program prefix location
; Free environment block
	PUSH	ES
	 MOV	SI,002CH
	 MOV	AX,ES:[SI]
	 MOV	ES,AX
	 MOV	AH,DOS_FREE_MEM
	 INT	DOS_INT
	POP	ES

IF 0
; Choose default printer (LPT1 on PC, PRN on Z-100)
; Disabled because on PC it has effect of resetting SETEGAD-set port #!
	MOV	AX,DEFAULT_PRN	; On the Z-100 it forces PRN_UN to zero.
	AND	TSR.PRN_UN,AX
ENDIF

; Checksum -- verify integrity of banner, copyright, etc.

	sub	bx,bx		; Clear counters
	mov	ax,bx
	mov	SI,OFFSET DGROUP:COPYRIGHT_MSG - 27
	call	crc
	mov	SI,OFFSET DGROUP:VERSION_CRC - 27
	call	crc

; INT 3			; !!!		; BX = number below, determined by stopping here
	SUB	BX,CRC_VALUE - 80h	; 80h is cmd-line char. count offset.
;
; HACKER DETERRENT... WE HOPE THIS STOPS FOLKS FROM MESSING WITH COPYRIGHT ETC.
; Important thing here (after checksums) is to have SI = 0080h to point to
; our command line.  We then compute '0' from this & put it into 'install.'
; If either calculation is incorrect the program won't run!
;
	MOV	SI,BX		; Put into SI
	xchg	bh,bl		; BX = 8000h
	add	bx,bx		; Now BX should be 0!
	or	bl,bh
	mov	install,bl	; Should be zero for proper operation

; Collect command line arguments.
;	MOV	SI,0080H	; Command line character count offset
	MOV	AL,ES:[SI]	; Get the character count
	MOV	CMD_POINTER,SI	; Save current command line location
	CLEAR	AH
	MOV	BX,AX
	INC	BX
	MOV	ES:[SI+BX],AH	; Mark end with null
; Process item, get next item, test for end loop.
START_LOOP:
	CALL	GET_NEXT	; Returns character in AL
	OR	AH,AH		; End of line if AH=0
	JNZ	START_CMD	; Nonzero: process command
	JMP	START_END_LINE	; Zero: Processed entire command line.

START_CMD:
	CMP	AH,TK_SWITCH
	JZ	START_SELECT_SWITCH
	JMP	START_SHOW_HELP

START_SELECT_SWITCH:
	CALL	GET_NEXT
	OR	AH,AH
	JNZ	START_ALPHA_TEST
	JMP	START_END_LINE

START_ALPHA_TEST:
	CMP	AH,TK_ALPHA
	JZ	START_OP_SELECT
	JMP	START_SHOW_HELP
;
;	Determine opcode after switch character
;
START_OP_SELECT:
	MOV	SI,OFFSET DGROUP:OPCODE_TABLE
	MOV	CL,AL		; Put opcode in CL
	NOT	CL		; So user can't examine .EXE file &
				; determine valid switches
OP_SEL_LOOP:
	LODSB			; Get opcode character
	OR	AL,AL		; Is it 0?
	JZ	OP_SEL_NONE
	MOV	BL,AL		; (save in BL)
	LODSW			; Get processor address
	CMP	BL,CL
	JNE	OP_SEL_LOOP
	sub	cx,cx		; Start with CX=0
	sub	dx,dx		;	 and DX=0
	JMP	AX		; Jump to processor at (AX)

OP_SEL_NONE:
	JMP	START_SHOW_HELP
;
;	/T - Terminate & Stay Resident
;
START_T:
	mov	al,OP_TSR		; Install as TSR
	jmp	short start_set_op	; Set opcode
;
;	/U - Un-Install memory-resident copy
;
START_U:
	mov	al,OP_REMOVE		; Remove TSR
	jmp	short start_set_op	; Set opcode
;
;	/P - Print Immediate (do not TSR)
;
START_P:
	mov	al,OP_PRINT
	; jmp	start_set_op
;
;	/T, /U, /P come here: Set the operation code.
;	Ensure user only specifies one operation.
;
START_SET_OP:
	cmp	INSTALL,OP_NONE
	jne	start_op_1
	MOV	byte ptr INSTALL,AL
	JMP	START_LOOP
START_OP_1:
	JMP	START_TERSE_HELP
;
;	/D - Device
;
START_D:
;	CMP	DEFAULT_PRN,0	; Z-100 printer must be 0!
;	JE	START_D_NG	;   Can't change it.
	CALL	GET_NEXT	; Look for = or blank etc.
	CMP	AH,TK_MISC
	JE	START_D
	CMP	AH,TK_ALPHA
	JE	START_D_1
START_D_NG:
	JMP	START_SHOW_HELP
START_D_1:
	MOV	BX,OFFSET TSR.PRN_UN	; Variable to set
	MOV	DI,OFFSET DGROUP:NAME_COM
	CALL	NEAR PTR COMPARE_WORD
	JZ	START_D_COM		; found COM...
	MOV	DI,OFFSET DGROUP:NAME_LPT
	CALL	NEAR PTR COMPARE_WORD
	JZ	START_D_LPT		; found LPT...
	JMP	START_D_NG		; found neither

START_D_LPT:			; BX already points to PRN_UN
	MOV	CX,1		; LPT0 illegal
	MOV	DX,MAX_PARPORT	; must be valid (LPT1..3 on PC)
	JMP	START_NUM	; Simply read a number from the cmd line

; Serial ports are saved as negative numbers!
START_D_COM:
	CALL	GET_NEXT	; find port number
	CMP	AH,TK_NUMBER
	JNE	START_D_NG	; must be COMn: or LPTn:
	MOV	CX,TOKEN_VALUE
	JCXZ	START_D_NG	; LPT0, COM0 illegal
	CMP	CX,MAX_SERPORT	; must be valid (COM1..4 on PC, COM1..2 on Z100)
	JA	START_D_NG
	NEG	CX		; Negate value to indicate COM port
				; fall through to START_D_SAVE:
START_D_SAVE:
	MOV	[BX],CX
	JMP	START_LOOP
;
;	/R - Region
;
START_R:
	MOV	CX,4		; Modify 4 entries
	SUB	BX,BX		; Beginning @ index=0
START_R_LOOP:
	CALL	GET_NEXT
	CMP	AH,TK_NUMBER
	JZ	START_R_1
	CMP	AH,TK_MISC
	JZ	START_R_LOOP
	JMP	short to_help ; START_SHOW_HELP
START_R_1:
	MOV	DX,TOKEN_VALUE
	INC	DX		; dump uses coordinates starting at 1

	push	bx		; save index
	add	bx,offset dgroup:tsr.x1 ; BX = address = Index + base
	mov	[bx],dx		; save value
	pop	bx		; Restore index

	INC	BX		; go to next coordinate entry
	INC	BX
	LOOP	START_R_LOOP
	JMP	START_LOOP
;
; Sets an entry in *ALL* color tables.
; ENTRY:  CX = Table index (0..15)
;	  DX = Color to set
;
SET_1_COLOR:
	PUSH	BX

	MOV	BX,CX
	ADD	BX,BX		; Make a word index
	MOV	PALETTE_TEXT[BX],DX
	MOV	PALETTE_16[BX],DX
	CMP	CL,4
	JGE	SET_1_A
	MOV	PALETTE_4[BX],DX
	CMP	CL,2
	JGE	SET_1_A
	MOV	PALETTE_2[BX],DX

SET_1_A:
	POP	BX
	RET
;
;	/C - Color Set
;
START_C:
;	xor	cx,cx		; Setting color 0 first
C_LOOP:
	CALL	GET_NEXT
	CMP	AH,TK_NUMBER	; Number -- go to it
	JE	START_C_1
	CMP	AH,TK_MISC	; Ignore misc.
	JE	C_LOOP
	CMP	AH,TK_ALPHA	; Error if alpha here
	JE	start_c_text	; (was TO_HELP)
	CMP	AH,TK_SWITCH	; Another switch?
	JNE	TO_START	; End_data -- exit loop
	JMP	START_SELECT_SWITCH ; Switch -- handle
TO_START:
	JMP	START_LOOP

START_C_TEXT:
	MOV	DI,OFFSET DGROUP:TEXT_REV ; is it "REV" ?
	call	compare_word
	jz	start_c_reverse
	jmp	to_help			; If not, don't allow.
start_c_reverse:
	or	tsr.tsrflags,TSRF_REVCOLOR ; set REVERSE COLOR flag
	jmp	start_loop

; Fill from current entry to end of table with this value
START_C_1:
	PUSH	CX
	MOV	DX,TOKEN_VALUE
START_C_2:
	CMP	CL,15		; Modify up to entry 15
	JG	START_C_3
	CALL	SET_1_COLOR
	INC	CX
	JMP	START_C_2
START_C_3:
	POP	CX
	INC	CX		; Address next entry
	JMP	C_LOOP
TO_HELP:
	JMP	START_SHOW_HELP	; Error -- show help message

;
;	/$=x	General parameter check.
;		Returns information in ERRORLEVEL parameter (child exit code.)
;		x=0	Return 1 if EGAD memory-resident
;		x=1	Return Printer Manufacturer code (0..31)
;		x=2	Return Printer Model code	 (0..15)
;		x=3..9	Return device capability/type bits:
;			3	Color Graphics
;			4	Color Text
;			5	Laser
;			6	Matrix (9 or 24 pin)
;			7	24-Pin Matrix
;			8	Inkjet
;			9	(reserved)
START_CHECK:
	CALL	GET_NEXT
	CMP	AH,TK_NUMBER
	JZ	START_CK_1
	CMP	AH,TK_MISC
	JZ	START_CHECK
	JMP	short to_help ; START_SHOW_HELP
START_CK_1:	

	mov	bx,token_value
	or	bl,bl
	jz	start_ck_inst
	dec	bx
	jz	start_ck_manuf	; 1 = check manufacturer
	dec	bx
	jz	start_ck_model	; 2 = check model
	cmp	bl,(9-2)
	jle	start_ck_devbits ; 3..9 return device capability bits
	mov	al,-4[bx]	; /$=10 returns data from offset 4 {Struc. size}
	jmp	exit_start_code	; and return it

start_ck_inst:			; check if EGAD installed
 IFNDEF OEM
	call	is_installed
	mov	ax,0100h	; Return 01 if loaded, 00 elsewise.
 ELSE
	mov	ax,0202h	; Always return 2 -- can't load TSR
 ENDIF
	jmp	short start_ck_zfl
	
start_ck_manuf:
	mov	al,byte ptr flag1
	and	al,31
	jmp	short start_ck_exit

start_ck_model:
	mov	ax,flag1
	mov	cl,5
	ror	ax,cl
	and	al,15
	jmp	short start_ck_exit

start_ck_devbits:
	mov	ax,512/2	; first 'device' bit in flag word
	mov	cl,bl		; (/2 since /$=3 gives us AX=1 here)
	rol	ax,cl
	test	flag1,ax
	mov	ax,0001h
start_ck_zfl:
	; Return AL if Zero flag, AH if NOT ZERO flag.
	jnz	start_ck_exit
	mov	al,ah		; Return alternate value
start_ck_exit:
	jmp	exit_start_code	; Return code in AL

;
;	/A=x	Select Alignment (default: 0)
;		x=1	Left (default)
;		x=2	Right
;		x=3	Center
START_A:
	MOV	BX,OFFSET TSR.CENTERING ; Variable's address
start_num13:
	MOV	CL,1		; Lower limit=1
start_ul_3:
	MOV	DL,3		; Upper limit=3
	JMP	short START_NUM	; Read numeric argument
;
;	/N=x	Select Alignment (default: 0)
;		x=1	Top (default)
;		x=2	Bottom
;		x=3	Center
START_N:
	MOV	BX,OFFSET tsr.vcentering ; Variable's address
	jmp	start_num13
;
;	/B=x	Turn crop Box on/off (default is ON)
;
START_B:
	MOV	DL,1		; Upper limit=1
	MOV	BX,OFFSET TSR.BOX ; Variable's address
	JMP	short START_NUM	; Read numeric argument
;
;	/E - Enlargement/Reduction
;
START_E:
	MOV	DL,5		; Upper limit=5 (5=enlarge 1.5x)
	MOV	BX,OFFSET TSR.ENLARGE ; Variable's address
	JMP	short START_NUM	; Read numeric argument
;
;	/V - Set VGA/256 Divisor
;
START_V:
	INC	CL		; Lower limit=1
	MOV	DL,128		; Upper limit=256
	MOV	BX,OFFSET TSR.VGA_DIVISOR ; Variable's address
	JMP	short START_NUM	; Read numeric argument
;
;	/F - Form-feed after printing?
;
START_F:
	MOV	BX,OFFSET TSR.DO_FF ; Variable's address
	jmp	start_ul_3	; Upper limit=3
;
;	/O=x	Set Orientation
;		x=0	Auto (default)
;		x=1	Portrait
;		x=2	Landscape
;
START_O:
	MOV	BX,OFFSET TSR.ORIENT ; Variable's address
	JMP	short START_NUM2 ; Read numeric argument (0..2)
;
;	/G=x	Set force graphics mode
;		x=0	Normal (default)
;		x=1	FORCE_TEXT
;		x=2	FORCE_GFX
;
START_G:
	MOV	BX,OFFSET FORCE_MODE ; Variable's address
	; fall thru:
START_NUM2:
	MOV	DL,2		; Upper limit=2
	; fall thru:
;
;	Get numeric argument.
;	BX points at word into which value will be placed.
;	Lower limit is in CX.
;	Upper limit is in DX.
START_NUM:
	CALL	GET_NEXT	; look for number, ignoring misc.
	CMP	AH,TK_MISC
	JE	START_NUM
	CMP	AH,TK_NUMBER
	JE	START_NUM_1
	JMP	short START_SHOW_HELP	; other than number or misc. found

START_NUM_1:
	MOV	AX,TOKEN_VALUE	; value in AX
	CMP	AX,CX
	JL	START_NUM_ERR
	CMP	AX,DX
	JLE	START_NUM_2	; must be in range CX..DX
START_NUM_ERR:
	JMP	short START_SHOW_HELP

START_NUM_2:
	MOV	[BX],AX		; Save value
	JMP	START_LOOP

START_TOPMSGS:
	DISPLY	COPYRIGHT_MSG	; Display copyright
	DISPLY	PRINTER_MSG
	DISPLY	VERSION_MSG
	RETN

START_HELP:
	CALL	START_TOPMSGS
	DISPLY	LONG_HELP1_MSG
	DISPLY	SHORT_HELP2_MSG
	DISPLY	LONG_HELP2_MSG
EXIT_START:
	mov	al,byte ptr errcod ; Return code
EXIT_START_CODE:
	MOV	AH,DOS_EXIT	; Function 4C (terminate process)
	INT	DOS_INT		; Goodbye!

;
;
START_END_LINE:

	mov	al,tsr.vcentering	; If vertical alignment is CENTERED,
	cmp	al,JUST_CENTER		; force RESET BEFORE if available.
	jne	s_e_l0
	or	tsr.do_ff,PG_RESET_BEFORE
s_e_l0:	MOV	AL,INSTALL	; AL = INSTALL, action code
	CMP	AL,OP_PRINT	; Begin printing?
	JE	START_PRINT
 IFNDEF OEM
	CMP	AL,OP_REMOVE	; Remove from memory?
	JE	START_REMOVE_TSR
	CMP	AL,OP_TSR	; Install as TSR?
	JE	START_INSTALL_TSR
 ENDIF
;	JMP	START_SHOW_HELP	; None selected.

START_SHOW_HELP:
	CALL	START_TOPMSGS
	DISPLY	SHORT_HELP_MSG
START_TERSE_HELP:
	DISPLY	TERSE_HELP_MSG
 IFNDEF OEM
	CALL	IS_INSTALLED	; Tell user whether EGAD is installed
	JNZ	TERSE0
START_ALREADY_INST:
	MOV	CX,OFFSET DGROUP:WAS_INST_MSG
	CALL	MEM_MSG
 ENDIF
TERSE0:	JMP	EXIT_START

START_PRINT:
; Print screen and exit
	CALL	RUN_DUMP
	JMP	short EXIT_START

IFNDEF OEM
 ;
 ; The 'OEM' version of the driver does not include the ability
 ; to terminate & stay resident.
 ;

 MEM_MSG:
	PUSH	CX
	DISPLY	STARS_MSG
	DISPLY	MEMRES_MSG
	POP	DX		; Passed in message
	DISPLY
	DISPLY	STARS_END_MSG
	RETN

 ;
 ;	/U - Un-Install memory-resident copy
 ;
 START_REMOVE_TSR:
	CALL	IS_INSTALLED
	JNE	REMOVE_NO	; Not installed, can't remove!

	CALL	REMOVE_TSR
	MOV	CX,OFFSET DGROUP:REMOVED_MSG
	CALL	MEM_MSG
	JMP	EXIT_START

 REMOVE_NO:
	MOV	CX,OFFSET DGROUP:NOT_INST_MSG
	CALL	MEM_MSG
	JMP	EXIT_START

 START_INSTALL_TSR:
	MOV	BX,EXEHDR	; Save PSP of this program
	MOV	INST_MCB,BX	; Save PSP for later removal

	push	ds
	 mov	ax,_bios
	 mov	ds,ax
	 ASSUME	DS:_bios
	 mov	byte ptr busy_flag,not_busy	; Reset BIOS busy flag
	pop	ds
	ASSUME	DS:DGROUP

	CALL	START_TOPMSGS
; Test for prior installation.
	CALL	IS_INSTALLED
	JNE	START_INST_TSR
; -- You can call REMOVE_TSR rather than print a message... but then the
; memory occupied by the previous instance often hangs around unused...
;	call	remove_tsr		; Remove previous copy
	JMP	START_ALREADY_INST	; Already installed, inform user & exit.

;
; Install into System Interrupt 5 (Print-Screen)
; DS:DX is address of interrupt handler
;
 START_INST_TSR:

	MOV	DX,offset DGROUP:INT_5
	MOV	AX,DOS_SET_VEC*256+PRTSC_INT
	INT	DOS_INT

	MOV	CX,OFFSET DGROUP:INST_MSG
	CALL	MEM_MSG
IF 1
; Copy printer name into PSP at offset 80h.
	mov	ax,exehdr
	mov	es,ax
	mov	di,80h		; Overwrite switch line
	mov	si,offset dgroup:printer_msg
	mov	cx,40		; Plenty of characters
	rep	movsb
ENDIF
;
; Terminate & Stay Resident
;
	MOV	DX,START_TEXT	; Start	of throw away part (end of keep part)
	MOV	AX,EXEHDR	; Start of PSP (Keep)
	SUB	dx,ax		; Number of segments to keep
	inc	dx		; round up

	MOV	AX,3100H	; Keep Process
	INT	DOS_INT		; terminate but	stay resident
				; This never returns (like Charlie on the MTA?)
;
;	Test for prior installation.
;	Returns:
;		'Equal' if driver is installed, 'Not Equal' if not installed.
;		ES:BX is current Interrupt 5 vector
;		      (ES:BX copied into OLD_INT_5 ptr.)
;
 IS_INSTALLED PROC NEAR
	MOV	AX,DOS_GET_VEC*256+PRTSC_INT	; Get INT 5 vector
	INT	DOS_INT

	MOV	WORD PTR OLD_INT_5,BX
	MOV	WORD PTR OLD_INT_5+2,ES

	; Installed driver must:
	; - Support Microsoft Interrupt convention
	; - Have EGAD flag word

	CMP	ES:WORD PTR [BX+6],INT_SIGNATURE
	JNE	IS_INST0		; Not recognized driver.

	PUSH	BX
	 CLEAR	BX			; 'QZ' should be @ offset 0 in segment
	 MOV	AX,WORD PTR FLAG
	 CMP	WORD PTR ES:[BX],AX
	POP	BX
 IS_INST0:
	RET
 IS_INSTALLED ENDP

 REMOVE_TSR PROC NEAR

	; NOTE! Versions prior to v2.41 had 10 header items.  When this was
	; changed to 12, the order was changed, and the INST_MCB field was moved
	; into the header. - WL 12/08/92

	; Rather than using "hdr_nvars" directly, we calculate its position from
	; the pointers in memory.  This should allow compatibility with future
	; versions in which the Configuration block is changed. - wl 12/09/92

	mov	di,SIG_LOC		; Point at signature
	mov	si,es:[di]		; get it ( = size of Config block)
	add	di,PAL_LEN+2		; Add length of palette
	add	si,di			; SI -> hdr_nvars

	cmp	byte ptr es:[si],12	; Require header to be compatible ( >= ours)
	jl	START_CANT_REMOVE	; If not, error!

	PUSH	ES
	PUSH	DS		; Set INT 5 to old vector (in DS:DX)
	 mov	dx,ES:WORD PTR [BX+2]
	 mov	ds,ES:WORD PTR [BX+4]
	 MOV	AX,DOS_SET_VEC*256+PRTSC_INT
	 INT	DOS_INT
	POP	DS
	POP	ES

 ; Release resident program's block
	MOV	AX,ES:WORD PTR INST_MCB	; PSP of installed program
	MOV	ES,AX

	MOV	AH,DOS_FREE_MEM		; Release memory block
	INT	DOS_INT
	JC	START_CANT_REMOVE	; Can't free it!  Internal error!
	RET
 REMOVE_TSR ENDP

 START_CANT_REMOVE:
	DISPLY	CANT_REM_MSG
	DISPLY	MEMRES_MSG
	; fall through
ENDIF

COMPARE_WORD PROC NEAR
	PUSH	ES
	MOV	AX,DS	; for string comparison we need ES=DS
	MOV	ES,AX
	MOV	SI,OFFSET DGROUP:TOKEN
	MOV	CX,3
	REP	CMPSB	
	POP	ES
	RET
COMPARE_WORD ENDP

CRC	PROC NEAR
	add	si,27
crc_loop:
	lodsb			; Get byte from copyright message
	neg	al		; XOR with 255
	cmp	al,0DCh		; negative '$'
	je	crc_exit
	rol	bx,1		; So it's position sensitive
	add	bx,ax
	jmp	crc_loop
crc_exit:
	RET
CRC	ENDP

$DISP	PROC	NEAR
	MOV	AH,9
	INT	21H
	RET
$DISP	ENDP
;
; End of installation section
;
	SUBTTL	Main Program (Command Line/Token Parsing)
	PAGE	+

;	GET_NEXT
;
;	Get and classify next command line argument
;	Entry:
;	  CMD_POINTER at next character
;	Returns:
;	  Copies token characters into TOKEN
;	  If token is numeric, sets value in TOKEN_VALUE
;	  Token type in AH:	0: LINE END
;				1: SWITCH
;				2: NUMBER
;				3: ALPHA
;				4: MISC
GET_NEXT PROC NEAR
	PUSH	DX
	PUSH	CX
	PUSH	BX
	LES	SI,DWORD PTR CMD_POINTER
	MOV	DI,OFFSET DGROUP:TOKEN
	SUB	AX,AX
	MOV	BYTE PTR [DI],AL ; Clear TOKEN
	MOV	TOKEN_VALUE,AX	; Clear TOKEN_VALUE
GET_SKIP_BLANKS:
	CALL	NEXT_CHAR	; Returns next character in AL
	OR	AL,AL
	JZ	GET_EOL		; No more char's on line.
	MOV	BX,OFFSET DGROUP:WHITE_SPACE	; Point at first parsing string
	CALL	ISIN		; Compare to white space
	JNZ	GET_SKIP_BLANKS	; Skip over blank spaces
	CALL	ISIN		; Compare to switch char's
	JNZ	GET_SWITCH
	CALL	ISIN		; Compare to digits	
	JNZ	GET_NUMBER
	CALL	ISIN		; Compare to alpha's
	JNZ	GET_ALPHAS
GET_MISC:
	MOV	AH,TK_MISC	; Must be misc. character (fall thru)
GET_EXIT:
	MOV	AL,TOKEN	; Return first character
	MOV	BYTE PTR [DI],0	; Put null at end of token
	MOV	CMD_POINTER,SI	; Save command line location
	POP	BX
	POP	CX
	POP	DX
	RET

GET_EOL:
	DEC	SI		; Point back at end of line
	CLEAR	AH		; no token
	JMP	GET_EXIT	; Zero in ah shows end of token set

GET_SWITCH:
	MOV	[DI],AL		; Process switch...
	INC	DI
	MOV	AH,TK_SWITCH
	JMP	short GET_EXIT

GET_NUMBER:			; Process numbers
	mov	ah,TK_NUMBER
	mov	cx,offset DGROUP:DIGITS
	jmp	short GET_WORD
GET_ALPHAS:
	mov	ah,TK_ALPHA
	mov	cx,offset DGROUP:ALPHAS
	; fall through

GET_WORD:
	mov	[di],al		; Put char into TOKEN buffer
	inc	di		; & advance

	cmp	ah,TK_NUMBER
	jne	GET_WORD1
	; Process number by accumulating
	push	ax
	 and	al,0fh		; Convert ASCII char to binary 0..9
	 cbw			; Make it a word
	 mov	bx,ax		; Save in BX
	 MOV	AL,10		; (AH already 0)
	 MUL	WORD PTR TOKEN_VALUE ; multiply old value by 10 (into DX:AX)
	 ADD	AX,BX		; accumulate value
	 MOV	TOKEN_VALUE,AX
	pop	ax

GET_WORD1:
	CALL	NEXT_CHAR	; Next character in AL
	OR	AL,AL		; Zero if end of line
	JZ	GET_WORD_DONE

	MOV	BX,CX		; Address of comparison table
	PUSH	AX		; Preserve token type in AH
	 CALL	ISIN		; Look for more letters or digits
	 OR	AH,AH		; If AH=0, char isn't of this type
	POP	AX
	JNZ	GET_WORD
GET_WORD_DONE:
	; At exit, AH contains TK_NUMBER or TK_ALPHA.
	DEC	SI		; Give back unused character
	JMP	short GET_EXIT	; and return

GET_NEXT ENDP
;
; ISIN
; Input:	AL =  character to find
;		BX -> string to find char. in
; Output: If character found:
;		'Not Zero'
;		AH =  character
;	  If not found:
;		'Zero'
;		AH =  0
;		BX -> next parse string
ISIN PROC NEAR
	xchg	ah,al		; Char to find in AH
ISIN_LOOP:
	MOV	AL,[BX]
	call	to_upper
	INC	BX
	OR	AL,AL		; End of string
	JZ	ISIN_DONE
	CMP	AH,AL
	JNZ	ISIN_LOOP
	OR	AH,AH		; Make 'Not Zero'
ISIN_DONE:
	xchg	ah,al
	RET
ISIN ENDP

NEXT_CHAR PROC NEAR
	INC	SI
	MOV	AL,ES:[SI]	; Get next command line character
	call	to_upper
	MOV	ES:[SI],AL
	RET
NEXT_CHAR ENDP

TO_UPPER PROC NEAR
	CMP	AL,'a'-1	; Just below 'a'
	JBE	NOT_LOWER
	CMP	AL,'z'+1
	JAE	NOT_LOWER
	SUB	AL,'a'-'A'
NOT_LOWER:
	RET
TO_UPPER ENDP

START ENDP
START_TEXT ENDS

	SUBTTL	Main Program (Initialization Data Segment)
	PAGE	+
;
;	This data is in memory only during invocation.
;	It is purged when EGAD is TSR.
;
START_DATA SEGMENT
	EXTRN	COPYRIGHT_MSG:BYTE
	EXTRN	PRINTER_MSG:BYTE

START_DATA ENDS

STACK	SEGMENT

; Let's try putting some of these things in the Stack segment.
; Can we overlay them with our interrupt stack?

	org	0

; *** Version info.
;	This will be checksummed.
VERSION_MSG	LABEL	BYTE
	DB	'   8139 E. Mawson Rd.',9,9
	version_info <>
VERSION_CRC	LABEL	BYTE
	DB	13,10
	DB	'   Mesa, AZ 85207',9,'602/380-9175',13,10

; *** Install-time data
;
;	OP_NONE   No operation selected
;	OP_TSR    Install (TSR)
;	OP_PRINT  Print only
;	OP_REMOVE Remove (/U)
;

IFDEF COMPANY_NAME
	DB	'Licensed for distribution by: '
	; Space for up to 29 characters of company name.
	; Will be overwritten later by parsed tokens.
	DB	COMPANY_NAME
	DB	13,10
	DB	'$'
TOKEN	DB	10 DUP (0)
ELSE
	DB	'$'
TOKEN	DB	20 DUP (0)
ENDIF
	; CAUTION! The 'TOKEN' buffer may overflow, corrupting
	; data which follow.
TOKEN_VALUE	DW	0
INSTALL		DB	0

; *** 
CMD_POINTER	DW	0
EXEHDR		DW	0	; Location of program header

; opcode table was here 09/15/94

; Parsing tables must be SPACES, SWITCHES, DIGITS, ALPHAS.

WHITE_SPACE 	DB	32,9,0				; tab & space
SWITCHES	DB	'/','-',0			; Switch indicators
DIGITS		DB	'6023809175',8,'4',0		; Numbers

IFDEF OEM
;			Legal switches are as follows:
;			'ACDEFGNOPRV',0
;			And we need the letters 'COM', 'LPT', and 'REV'. But
;			confuse the issue by including the letters in the
;			following message. '$' needed for '/$' (Spaces ignored)
ALPHAS		DB	'EGAD Copyright (c) LS Software$D'
;		DB	'B'				; crop box fn disabled
TEXT_REV	DB	'REV'
		DB	'NVM'
NAME_LPT	DB	'LPT'
		DB	0

ELSE
ALPHAS		DB	'EGAD (C) LS Software$?ChinBumpQVN',0 ; Alphabetics (and '?')
ENDIF

; Create an error if these data exceed space set aside for stack. 
; Also allow at least 20 bytes for startup stack. WL 09/15/94
.ERRNZ	$ GT (INT_STACK-20)

STACK	ENDS

START_DATA	SEGMENT

; Create opcode table.  Perform operation on characters to disguise table.
OPCODE	macro	letter,address
	DB	(letter XOR 255)
	DW	address
	endm

OPCODE_TABLE LABEL BYTE
	OPCODE	'A',START_A
IFNDEF OEM
	OPCODE	'B',START_B
ENDIF
	OPCODE	'C',START_C
	OPCODE	'D',START_D
	OPCODE	'E',START_E
	OPCODE	'F',START_F
	OPCODE	'G',START_G
	OPCODE	'N',START_N
	OPCODE	'O',START_O
	OPCODE	'P',START_P
	OPCODE	'R',START_R
	OPCODE	'V',START_V
IFNDEF OEM
	OPCODE	'T',START_T	; Omit TSR support in OEM version
	OPCODE	'U',START_U

	OPCODE	'H',START_HELP	; Omit help messages in OEM version
	OPCODE	'?',START_HELP
ENDIF
	OPCODE	'$',START_CHECK
	DB	0	; end of table

IFDEF OEM

  IF1
  %OUT *** Creating OEM Version ***
  ENDIF

  ;
  ; OEM version includes only this help message:
  ;
  SHORT_HELP_MSG	LABEL BYTE
  LONG_HELP1_MSG	LABEL BYTE
	DB	13,10
NAME_COM LABEL BYTE	; Following 3 letters must be "COM"
	DB	'COMPLETE documentation, as well as a version of EGAD which',13,10
	DB	'may be loaded as a TSR (Terminate & Stay Resident), may be obtained',13,10
	DB	'from LS Software.  Call or write for a free catalog.',13,10
  SHORT_HELP2_MSG	LABEL BYTE
  LONG_HELP2_MSG	LABEL BYTE
  TERSE_HELP_MSG	LABEL BYTE
  STARS_MSG		LABEL BYTE
  STARS_END_MSG		LABEL BYTE
  STARS_ERR_MSG		LABEL BYTE
  MEMRES_MSG		LABEL BYTE
  REMOVED_MSG		LABEL BYTE
  NOT_INST_MSG		LABEL BYTE
  WAS_INST_MSG		LABEL BYTE
  INST_MSG		LABEL BYTE
		DB '$'
  MSG_INT_ERR   DB	7,'Internal Error',13,10,'$'
ELSE
  MSG_INT_ERR   DB	7,'*** Internal Error'
  STARS_END_MSG	DB	' ***',13,10,'$'

  STARS_MSG	DB	'*** $'
  STARS_ERR_MSG	DB	'*** ERROR! $'
  MEMRES_MSG	DB	'Memory-Resident EGAD $'
  REMOVED_MSG	DB	'was Removed.$'
  NOT_INST_MSG	DB	'Not Installed, ',7	; falls through...
  CANT_REM_MSG	DB	'Cannot Remove ','$'
  WAS_INST_MSG	DB	'Already Installed.','$' ; removed BELL
  INST_MSG	DB	'Installed ***',13,10
		DB	'*** May be activated by pressing Print-Screen.$'

  SHORT_HELP_MSG LABEL BYTE
	DB	10
	DB	'EGAD /T|/P|/U	'
  SHORT_HELP2_MSG LABEL BYTE
	DB	'[One of /P, /T, /U must be specified:]',13,10
	DB	' /T		Installs as TSR on Shift-PrtSc key',13,10
	DB	' /P		Prints screen and exits',13,10
	DB	' /U		Un-Installs (Removes) memory-resident EGAD',13,10
	DB	'$'

  TERSE_HELP_MSG LABEL BYTE
	DB	13,10
	DB	'Use /HELP or /? to display complete help information.',13,10,13,10
	DB	'*** This command DID NOT activate EGAD! ***',13,10
	DB	'$'

  LONG_HELP1_MSG LABEL BYTE
	DB	13,10,'REQUIRED Switches: $'

  LONG_HELP2_MSG LABEL BYTE
;	DB	13,10
	DB	'OPTIONAL Switches: [NOTE: SETEGAD can adjust defaults.]		DEFAULT:',13,10
	DB	' /A=n		Print Alignment: n= 1:Left, 2:Right, 3:Centered	 1',13,10
	DB	' /B=n		n=0:Print immediately; 1:Adjustable crop-box.	 1',13,10
	DB	' /C=n,n,...	Selects printer colors. B&W printers: 0=Wht..4=Gray..7=Blk',13,10
	DB	'		Color:0=Wht,48=Yel,12=Mag,60=Red,3=Cya,51=Grn,11=Blu,63=Blk',13,10
	DB	' /C='
; Cheat a little bit by putting program strings within help message!
TEXT_REV LABEL BYTE
	DB	'REV		Forces reversed color table.',13,10
	DB	' /D=xxx		Device to use for output ('
NAME_COM LABEL BYTE
	DB	'COM1..4, '
NAME_LPT LABEL BYTE
	DB	'LPT1..3)	LPT1',13,10
	DB	' /E=n		n=0:Reduction; 1..4:Enlargements; 5:1.5x	 1',13,10
	DB	' /F=n		Format: n=0:none; 1:Eject page after printout;	 0',13,10
	DB	9,9,'2:Reset printer before printing; 3: (1) AND (2)',13,10
	DB	' /N=n		Print position: n= 1:Current, 2:Bottom, 3:Center 1',13,10
	DB	' /O=n		Orientation: n=0:auto, 1:portrait, 2:landscape	 0',13,10
	DB	' /R=x1,y1,x2,y2',9,'Upper-left & lower-right coordinates	    (Whole Screen)',13,10
	DB	' /V=n		Divisor for 256-Color modes			 1',13,10
	DB	'$'
ENDIF

xx1 db '#1$'

START_DATA ENDS

	END	START
