;
;	Copyright (C) 1988, 1989, 1990, 1991
;		LS Software
;		8139 E. Mawson
;		Mesa, AZ 85207
;		(602) 380-9175
;
;
; **********************************************************************
;
;	EGAD Screen Print
;	-----------------
;
;	Printer Specific Code
;
;	Contain control code sequences, printer graphic mode info,
;	bit mapping patterns, and printer specific dither tables.
;
;	X and Y numbers in graphic mode tables indicate the maximum
;	number of printable pixels. This is so that an enlarged image
;	could fill the entire printed page, but no more.
;
;	Dither Tables.
;
;	These bytes are AND'ed with the OR-bits from the pattern
;	table.  The index into this table is the color level (0..3) from
;	the printer color number.
;
;	SET_DOT uses these tables by referencing the pointer DITHER_PTR.
;	PRINTER_LINE counts from 0 to NDITHERS and puts the index into
;	DITHER_NUM, and sets the pointer in DITHER_PTR.

PRN_ID EQU PRN_TYP * 512 + PRN_MDL * 32 + PRN_MFG
	SHOWIT <PRN_ID>,%(PRN_ID)

_DATA	SEGMENT
	ORG	0

; Driver Identifier:
FLAG	DB	'QZ'	; Flag to find printer type & structure identity
FLAG1	DW	PRN_ID	; Printer identifier word

SIGNATURE DW	CFG_LEN	; Size of structure (in bytes)

; -----------------------------------------------------------
;	Settings changeable from SETEGAD:
;
;	******************** CAUTION!!! ********************
;
;	DO NOT CHANGE THE ORDER OF THE BELOW, AS THIS ORDER
;	IS ASSUMED BY SETEGAD!!!
;
; -----------------------------------------------------------
;
;	Color Numbers:
;
;	Color Printers:	  BBMMCCYY
;	 except Paintjet: --MMCCYY (Black if M,C,Y bits all true)
;	B & W Printers:   ----BBBB (dot matrix printers use bottom 2 bits)
;				 (laser	printers use 3 or 4)
;
IF (prn_typ AND typ_color) NE 0
	PALETTE_2 	DW	0,42
	PALETTE_4 	DW	0,3,12,42
	PALETTE_16 	DW	0,11,51,3,60,12,57,42
			DW	21,10,34,1,40,8,48,63
	PALETTE_TEXT 	DW	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 ; B&W text
ELSE
	PALETTE_2 	DW	0,7
  IF (prn_typ AND typ_matrix)
	PALETTE_4 	DW	0,2,4,7   ; Need somewhat lighter colors
  ELSE
	PALETTE_4 	DW	0,3,4,7   ; Darker colors are fine for lasers
  ENDIF
	PALETTE_16 	DW	0,1,2,3,4,5,6,7,4,1,2,3,4,5,6,7
	PALETTE_TEXT 	DW	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0
ENDIF

IF (prn_typ AND typ_laser)
TSR	CONFIG_DATA <1,1,1,1,1,1,1,1,1,0,0,0,0,15,1,1> ; Default to Form Feed
ELSE
TSR	CONFIG_DATA <>
ENDIF

; -----------------------------------------------------------
;	End of SETEGAD modifiable variables
;	The below may be used to change printer definitions
; -----------------------------------------------------------
hdr_nvars	DW	12				; # of words following:
hdr_version	DW	EGAD_VER			; EGAD Version #

; Ensure that INST_MCB is located where we can find it.  It should follow
; the QZxx + palette + config_block + header_above.
INST_MCB	DW	0				; PSP of TSR
.errnz (OFFSET INST_MCB)-MCB_LOC

hdr_nmap	DB	(GFXMAP_END-GMAP0)/(GMAP1-GMAP0) ; # of mappings
		db	N_TEXT_ATTRS			; # of text attributes
hdr_map0	DW	GMAP0				; Pointer to first map
hdr_secmap	DB	6				; 1st secondary mode #
hdr_byteorient	DB	prn_byte			; 0=Vertical, 1=Horizontal
hdr_nmodes	DW	MAXMODE+1			; # of printer gfx modes
hdr_mode0	DW	GMODES				; ptr to 1st gfx mode
hdr_pat0	DW	PATTERN_200-1			; ptr to first pattern
hdr_dith_n	DB	NDITHERS			; # of dither tables
hdr_iscolor	DB	((PRN_TYP AND TYP_COLOR) NE 0)	; color=1, B&W=0
hdr_dith_lev	DB	DITHER_LEVELS			; levels per table
hdr_ncolors	DB	NBUFFERS			; # of color buffers
hdr_dith_wid	DB	DITHER_WIDTH			; Bytes per level
		db	PRN_GR				; Graphics mode description
hdr_dith0	DW	DITHER_TABLE			; ptr to dither table
; Note: Number of actual printer colors in graphics modes is:
;	hdr_dith_lev to the hdr_ncolors power.
; -----------------------------------------------------------

IF PRN_MFG EQ MFG_HP
 ; HP PCL printers
 include drv_hp.inc
ENDIF

IF ((PRN_MFG EQ MFG_EPSON) AND (PRN_MDL LT MDL_EPLQ)) OR \
   ((PRN_MFG EQ MFG_IBM) AND (PRN_MDL EQ MDL_IBMPP))
 ; Epson and IBM 9-pin matrix printers
 include drv_ep9.inc
ENDIF

IF ((PRN_MFG EQ MFG_EPSON) AND (PRN_MDL GE MDL_EPLQ)) OR \
   ((PRN_MFG EQ MFG_IBM) AND (PRN_MDL EQ MDL_IBMX24))
 ; Epson and IBM 24-pin matrix printers
 include drv_ep24.inc
ENDIF

IF prn_mfg eq mfg_oki
 include drv_oki.inc
ENDIF

IF (prn_mfg eq mfg_DP)
 include drv_dp.inc
ENDIF

IF (prn_mfg eq mfg_NEC)
 include drv_nec.inc
ENDIF

IF (prn_mfg eq mfg_pcpi)
 include drv_pcpi.inc
ENDIF

IF (prn_mfg eq mfg_Adobe)
 include drv_post.inc
ENDIF

IF ((prn_mfg eq mfg_ibm) and (prn_mdl eq mdl_IBM3852))
 include drv_ibmj.inc
ENDIF

IF (prn_mfg eq mfg_MPI)
 include drv_mpi.inc
ENDIF

IF (prn_mfg eq mfg_Tandy)
 include drv_rs.inc
ENDIF

IF (prn_mfg eq mfg_Toshiba)
 include drv_tosh.inc
ENDIF

IF (prn_mfg eq mfg_Xerox) and (prn_mdl eq mdl_Xerox4020)
 include drv_xe.inc
ENDIF

IF (prn_mfg eq mfg_Canon)
 include drv_can.inc
ENDIF

if (PRN_CHRSET NE CHRSET_PC8)
	include	xlat.inc
ENDIF

IF (prn_mfg ne mfg_Adobe)
	; For all except PostScript printers
	EGADSTR FF,<12>	; Form feed (0-length unless turned on)
ENDIF

; Number of text attributes:
IFNDEF N_TEXT_ATTRS
	N_TEXT_ATTRS EQU 0
ENDIF

COLOR_TEXT = ((prn_typ AND typ_color) NE 0) AND (N_TEXT_ATTRS EQ 0)

;	****************************************
;
;	End of Printer-Specific Support Code
;
;	****************************************
;
IF PRN_GR EQ GR_QUO
	QUOTE_FLAG DB 0
ENDIF
IF PRN_GR EQ GR_BIT
	QUOTE_FLAG DB 0
ENDIF
IF PRN_GR AND GR_XHEX
	QUOTE_FLAG DB 0
ENDIF

; End of string	variables
_DATA	ENDS

	SUBTTL	Printer Name area
	PAGE +

START_DATA	SEGMENT	WORD PUBLIC 'INIT'

PRINTER_MSG LABEL BYTE
if (prn_mfg eq mfg_HP)
	DB	'HP '
  if (prn_mdl eq mdl_hplj)
	DB	'LaserJet'
  endif
  if (prn_mdl eq mdl_hplj2)
	DB	'LaserJet II'
  endif
  if (prn_mdl eq mdl_hplj3)
	DB	'DeskJet; HP LaserJet IIP/III/4'
  endif
  if (prn_mdl eq mdl_hpqj)
	DB	'QuietJet'
  endif
  if (prn_mdl eq mdl_hppj)
	DB	'PaintJet'
  endif
  if (prn_mdl eq mdl_hpdj500)
	DB	'DeskJet 500'
    if (prn_typ and typ_color)
	DB	'C'
    endif
  endif
  if (prn_mdl eq mdl_hpdj550)
	DB	'DeskJet 550C/560C'
  endif
endif
if (prn_mfg eq mfg_Epson)
  if (prn_mdl eq mdl_epSt)
	db	'Star Gemini/SG/SD/SR'
  else
	db	'Epson '
    If (prn_mdl eq mdl_epMX)
	db	'MX (120 dpi)'
    Else
      IF (PRN_TYP AND TYP_MAT24) EQ TYP_MAT24
	db	'LQ/Compatible (24Pin)'
      ELSE
	db	'& Compatibles (9-Pin)'
      ENDIF
    Endif
  endif
endif
if (prn_mfg eq mfg_IBM)
	db	'IBM '
    if (prn_mdl eq mdl_ibmpp)
	db	'24-Pin Printer, 9-pin mode'
    endif
    if (prn_mdl eq mdl_ibmx24)
	db	'ProPrinter X24 (24-pin)'
    endif
    if (prn_mdl eq mdl_ibm3852)
	db	'Color Inkjet 3852'
    endif
endif
if (prn_mfg eq mfg_Toshiba)
	db	'Toshiba '
	db	'P3x1, P1340/1350/1351'
endif
if (prn_mfg eq mfg_NEC)
	db	'NEC '
	db	'8023, C.Itoh 8510'
endif
if (prn_mfg eq mfg_DP)
	db	'DataProducts '
    if (prn_mdl eq mdl_dp8050)
	db	'8050'
    endif
    if (prn_mdl eq mdl_dptiger)
	db	'Paper Tiger'
    endif
endif
if (prn_mfg eq mfg_OKI)
	db	'Okidata '
   if (prn_mdl eq mdl_oki82)
	db	'82/83/84/92/93'
   endif
   if (prn_mdl eq mdl_oki292)
	db	'292 [ONLY]'
   endif
endif
if (prn_mfg eq mfg_MPI)
	db	'MPI-88G, MPI-150G'
endif
if (prn_mfg eq mfg_PCPI)
	db	'PCPI '
	db	'LaserImage 2000'
endif
if (prn_mfg eq mfg_Xerox)
	db	'Xerox 4020'
endif
if (prn_mfg eq mfg_Tandy)
	db	'Tandy '
   if (prn_mdl eq mdl_rsdmp105)
	db	'DMP-105'
   endif
   if (prn_mdl eq mdl_rscgp220)
	db	'CGP-220'
   endif
endif
if (prn_mfg eq mfg_Adobe)
	db	'PostScript'
endif
if (prn_mfg eq mfg_Canon)
	db	'Canon LBP'
endif

if (prn_typ and typ_color)
	db	' (Color)'
endif

ifdef WIDE
	db	13,10,9,9,9,9,9,9,'[Wide Carriage]'
endif

	DB	13,10
	DB	'$'

START_DATA	ENDS

